;
.include "Header_Wii_Nunchuk.h"     ;Label und Werte Zuweisungen
.include "Hardware_Wii_Nunchuk.h"   ;Hardware Label  Zuweisungen
;
;Programmstart nach RESET  ( Interrupt Vektoren Tabelle )
;
.CSEG                       ;Was ab hier folgt kommt in den FLASH-Speicher
.ORG $0000                  ;Programm beginnt an der FLASH-Adresse 0x0000

 rjmp  _reset               ;RESET-Vector

.include "IRQm8_Wii_Nunchuk.inc";Restliche Interrupt Vektortabelle

.ORG INT_VECTORS_SIZE       ;Programmadresse nach den ganzen IRQ-Vektoren

;
;IIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIII
;IIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIII
;I Erstinitialisierungen
;IIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIII
;IIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIIII
;
;Stapelzeiger initialisieren (fuer Unterprogramme und / oder Interrupts)
;
_reset:
 ldi  a,High(RAMEND)        ;RAMEND (SRAM) ist in der Definitions-
 out  SPH,a                 ;datei festgelegt  
 ldi  a,Low (RAMEND)
 out  SPL,a
;
;Variablen initialisieren ( r0-r31 haben unbestimmten Inhalt )
;
 ldi  anz_char,#anz_char    ;Anzahl der zu sendenden Zeichen
 clr  flag_reg              ;Allgemeines Flaggenregister
 clr  key_reg               ;Tasterentprellregister
 clr  ovf0_zaehler          ;Timer0 Overflowzaehler
;
;Ports Erstkonfiguration
; 
 ldi  a,$FF                 ;PortB kommplett als...
 out  DDRB,a                ;...Ausgang konfigurieren
 sbi  LED_PORT,led.ok       ;OK-LED ausschalten ( Vorerst Pullup )
 sbi  LED_DDR,led.ok        ;PORT-Bit als Ausgang f. OK-LED
 sbi  KEY_PORT,key.bit      ;Pullup f. Taster einschalten
;
; USART Baudrate einstellen
;
 ldi  a,#ubrr               ;f. UBRRH bei 38400 Baud mit 1,8432Hz Quarz
 clr  b                     ;f. UBBRL bei 38400 Baud mit 1,8432Hz Quarz
 out  UBRRH,b               ;Baudrate 38400 mit einem...
 out  UBRRL,a               ;...1,8432Hz Quarz
;
;USART Sender ( Transmitter ) freigeben sowie die
;Data Register Empty Interrupt 
;
; ldi  a,1<<TXEN | 1<<UDRIE  ;USART Transmit Enable & ...
; out  UCSRB,a               ;...USART Data Register Empty Interrupt        
;
;USART Format ( 8,N,1 ) ist bereits nach dem Reset voreingestellt
;
;
;Timer0 Overflow Interrupt initialisieren
;
 ldi  a,1<<TOIE0            ;Timer0 Overflow Interrupt freigeben...
 out  TIMSK,a               ;...
 ldi  a,1<<CS02             ;Timer0 Vorteiler => 256...
 out  TCCR0,a               ;...
;
;TWI SCL Frequenz einstellen ( ca. 102 kHz mit TWBR=2 ca. 92kHz )
;und Two-wire Interface Interrupt freigeben
 ldi  a,#twbr                ;Ergibt zusammen mit den Vorteilerbits...
 out  TWBR,a                 ;...TWPS=0 im TWSR ca. 102 kHz
 in  a,TWCR                 ;TWI Control Register lesen...
 ori a,1<<TWIE ;|1<<TWEN             ;...und TWI-Interrupt... 
 out TWCR,a                 ;...freigeben

 sei                        ;Globale Interruptfreigabe im CPU-Statusregister
;
;HHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHH
;HHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHH
;H Hauptprogrammschleife
;HHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHH
;HHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHHH
;
_main:
 sbrs key_reg,key.actuated  ;Taster betaetigt und entprellt?...
 rjmp _idle                 ;...Nein => Weiter im Progrmam
;
;TWI START-Bedingung mit Interruptfreigabe
;
 
 rcall _twi_start           ;TWI START-Bedingung mit Interruptfreigabe

;
;USART pro Tasterbetaetigung ein Byte senden
;
; sbis UCSRA,UDRE            ;USART Data Register frei?... 
; rjmp _idle                 ;...Nein => Weiter im Progrmam
; out  UDR,udr_char          ;Zu sendenes Zeichen in UDR hineinschreiben  

 cbr  key_reg,1<<key.actuated;Tasterbetaetigung quitieren

 rjmp _idle                 ;... Weiter im Progrmam  ...
 
            
_newchar:                   ;Gucken ob neues Zeichen vorhanden ist...
 tst  anz_char              ;Letztes Zeichen ueberschritten?
 brmi _transmit_off         ;Ja => springen...
 inc  char_dummy            ;...ansonsten Zeichen simulieren
 ;mov  udr_char,char_dummy   ;...neues Zeichen uebergeben... 
 subi anz_char,1            ;...Zeichenzaehler -1...
 rjmp _idle                 ;...C ruht bis ein neues Zeichen gesendet werden kann

_transmit_off:
 ldi  anz_char,#anz_char    ;Anzahl der zu sendenden Zeichen
 clr  char_dummy            ;Simulieren der Zeichen
_idle:                      ;C ruht bis ein neues Zeichen gesendet werden kann
 sbr  flag_reg,1<<pla.flg   ;Programmlaufanzeige-Flag quitieren

 rjmp _main                 ;Sprung zum Anfang des Hauptprogramms ( Mainloop )
; 
;UUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUU
;UUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUU
;U Beginn der verwendeten Unterprogramme
;UUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUU
;UUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUU
;
.include "UP_Wii_Nunchuk.inc";Unterprogramme
;
;QQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQ
;QQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQ
;Q Beginn der verwendeten Interrupt Service Routinen ( ISR )
;QQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQ
;QQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQQ
;
.include "ISR_Wii_Nunchuk.inc";Interrupt Service Routinen

.EXIT                       ;Ende des Quelltextes
